/*
 * File:   mainpicc.c
 * Author: family
 *
 * Created on 2011/05/14, 21:38
 */

#include "xc.h"

#include "7seg_p16f1823.h"
#include "HW_Config_p16f1823.h"
#include "sound.h"

#ifndef _XTAL_FREQ
#define _XTAL_FREQ 4000000	// 4MHz
#endif

#pragma config FOSC = INTOSC // INTOSCIO oscillator:
#pragma config MCLRE = OFF  // MCLR pin function is RA3
#pragma config PWRTE = ON   // PWRT enabled
#pragma config WDTE = OFF   // WDT disabled
#pragma config PLLEN = OFF   //
#pragma config BOREN = OFF  // BOR disabled
#pragma config BORV = LO    // BOR voltage
#pragma config LVP = OFF    // Low Voltage Programming disable

__EEPROM_DATA (0xE7, 0x03, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF);  // high score 0x3E7 = 999

unsigned char mask_goal, mask_start;
unsigned char flg_ioca;
unsigned int t_sec, t_sec2;
unsigned char t_sec10;
unsigned int best_score;

void goto_bed(void);
void display_score(int score);
void display_best();
void clear_best();
void read_best();

void delay_100ms(void)
{
    __delay_ms(100);
}

void delay_500ms(void)
{
    __delay_ms(500);
}

int main(void)
{
    unsigned char i;
    INIT_HW();
    SET7SEG(Soff);
    i = 0;
    while(!SWCH)
    {
        delay_100ms();
        if(++i > 30)
            display_best();
    }
    delay_100ms();

    while(1){
        SET7SEG(S_);

        IOCAF = 0;
        flg_ioca = 0;
        INTCON = 0b11001000;

        i = 0;
        while(1){
            delay_100ms();
            if(!START){
                mask_goal = MASK_GOAL;
                mask_start = MASK_START;
                break;
            }
            if(!GOAL){
                mask_goal = MASK_START;
                mask_start = MASK_GOAL;
                break;
            }
            if(++i > 250 || (flg_ioca & MASK_SW) != 0)
                goto_bed();
        }

        SET7SEG(S0);   // display "0"

        IOCAF = 0;
        flg_ioca = 0;
        INTCON = 0b11001000;
            //  bit7: GIE = 1;
            //  bit6: PEIE = 1;
            //  bit4: INTE = 1;
            //  bit3: IOCIE = 1;
            //  bit1: INTF = 0;
            //  bit0: IOCIF = 0;

        // wait for start
        i = 0;
        do{
            flg_ioca = 0;
            IOCIE = 1;
            delay_100ms();
            if(++i > 250 || (flg_ioca & MASK_SW) != 0)
                goto_bed();
        } while((flg_ioca & mask_start) != 0 || (PORTA & mask_start) == 0);

        //start
        SET7SEG(Soff);  // display off
        delay_500ms();
        if((flg_ioca & mask_start) != 0)
            break;
        
        IOCIF = 0;
        IOCAF = 0;
        flg_ioca = 0;
        IOCIE = 1;

        TMR1 = 0;
        TMR1IE = 0;
        TMR1ON = 1;

        for(t_sec=2; t_sec<2000; t_sec++){  // priod of t_sec = 500msec
            TMR1IF = 0;
            TMR1 += 336;    // TMR1 priod = 500msec
            if(((char)t_sec & 0x01) == 0){
                if(t_sec < 400){
                    t_sec2 = (t_sec >> 1) % 100;
                    t_sec10 = t_sec2 / 10 + 2;
                }else{
                    t_sec2 = t_sec / 20;
                    t_sec10 = t_sec2 % 10 + 2;
                }
                sound_on(note_ary[t_sec10]);
                SET7SEG(Seg7[t_sec2 / 10]);
                while(TMR1H<150){
                    if(flg_ioca)
                        break;
                }
                sound_off();
            }
            else
            {
                SET7SEG(Soff);
                while(TMR1H<50){
                    if(flg_ioca)
                        break;
                }
                SET7SEG(Seg7[t_sec2 % 10]);
                while(TMR1H<150){
                    if(flg_ioca)
                        break;
                }
                SET7SEG(Soff);
            }
            while(!TMR1IF){
                if(flg_ioca)
                    break;
                    //goto OUT;   //exit for(i)
            }
            if(flg_ioca)
                break;
            //if((PORT_SG & MASK_SG) != MASK_SG)
            //    goto OUT;   //exit for(i)
        }   // End of for(i)

        INTCON = 0;
        if((flg_ioca & mask_goal) != 0){
            // goal
            SET7SEG(SG);
            sound_on(NOTE5C);

            __delay_ms(1000);
            read_best();
            sound_off();
            t_sec2 = t_sec >> 1;
            for(i=0; i<3 || (PORT_SG & MASK_SG) != MASK_SG; i<0x0F? i++:i){
                display_score(t_sec2);
                /*
                SET7SEG(Soff);
                delay_500ms();
                SET7SEG(Seg7[t_sec2 / 100]);
                delay_500ms();
                SET7SEG(Soff);
                delay_100ms();
                t_sec2 = t_sec2 % 100;
                SET7SEG(Seg7[t_sec2 / 10]);
                delay_500ms();
                SET7SEG(Soff);
                delay_100ms();
                SET7SEG(Seg7[t_sec2 % 10]);
                delay_500ms();;
                */
            }
            //while(!(GOAL & START));
            if(t_sec2 < best_score){
                eeprom_write(0, (unsigned char)t_sec2);
                eeprom_write(1, (unsigned char)(t_sec2 >> 8));
                SET7SEG(SH);
                sound_pkmn();
            }

        }
        else if((flg_ioca & MASK_COURSE) != 0){
            //miss
            SET7SEG(S_);
            sound_on(NOTE4C);
            __delay_ms(2000);
            sound_off();
        }
        else if(flg_ioca == 0 || (flg_ioca & MASK_SW) != 0){
            //switch pressed
            goto_bed();
        }
    }   // End while(1)
    return 0;
}

static void interrupt isr(void)
{
    if(T0IF){
        SPK = !SPK;
        TMR0 += t0_add;
        T0IF = 0;
    }
    if(IOCIF){
        flg_ioca = IOCAF;
        IOCIE = 0;
        IOCIF = 0;
        IOCAF = 0;
    }
}

void goto_bed(void){
    SET7SEG(Soff);
    sound_off();
    delay_100ms();
    while(!SWCH);
    delay_100ms();
    IOCAN = MASK_SW;
    IOCAF = 0;
    IOCIF = 0;
    IOCIE = 1;
    SLEEP();
    NOP();
    delay_100ms();
    asm("RESET");

}

void display_best(void){
    unsigned char j = 0;
    SET7SEG(SH);
    while(!SWCH){
        delay_100ms();
        if(++j>50){
            SET7SEG(SC);
            sound_on_10ms(12, 100);
            eeprom_write(0, 0xE7);
            eeprom_write(1, 0x03);
            while(!SWCH);
            break;
        }
    }
    read_best();
    for(j=0; j<3; j++){
        display_score(best_score);
    }
    delay_100ms();
}

void read_best(){
    best_score = eeprom_read(1);
    best_score <<= 8;
    best_score += eeprom_read(0);
}

void display_score(int score){
    SET7SEG(Soff);
    delay_500ms();
    SET7SEG(Seg7[score / 100]);
    delay_500ms();
    SET7SEG(Soff);
    delay_100ms();
    score = score % 100;
    SET7SEG(Seg7[score / 10]);
    delay_500ms();
    SET7SEG(Soff);
    delay_100ms();
    SET7SEG(Seg7[score % 10]);
    delay_500ms();;
}